# coding=utf-8
# Copyright 2021 Open Business Software Solutions, The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Translation Edit Rate (TER) metric. The part of this file is adapted from HuggingFace's
datasets package implementation of TER metric. See
https://github.com/huggingface/datasets/blob/master/metrics/ter/ter.py
"""
from typing import Callable, Dict, Sequence

import datasets

from jury.collator import Collator
from jury.metrics import LanguageGenerationInstance, MetricForLanguageGeneration
from jury.metrics._core.utils import PackagePlaceholder, requirement_message

# `import sacrebleu as scb` placeholder
scb = PackagePlaceholder(version="2.0.0")


_CITATION = """\
@inproceedings{snover-etal-2006-study,
    title = "A Study of Translation Edit Rate with Targeted Human Annotation",
    author = "Snover, Matthew  and
      Dorr, Bonnie  and
      Schwartz, Rich  and
      Micciulla, Linnea  and
      Makhoul, John",
    booktitle = "Proceedings of the 7th Conference of the Association for Machine Translation in the Americas: Technical Papers",
    month = aug # " 8-12",
    year = "2006",
    address = "Cambridge, Massachusetts, USA",
    publisher = "Association for Machine Translation in the Americas",
    url = "https://aclanthology.org/2006.amta-papers.25",
    pages = "223--231",
}
@inproceedings{post-2018-call,
    title = "A Call for Clarity in Reporting {BLEU} Scores",
    author = "Post, Matt",
    booktitle = "Proceedings of the Third Conference on Machine Translation: Research Papers",
    month = oct,
    year = "2018",
    address = "Belgium, Brussels",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/W18-6319",
    pages = "186--191",
}
"""

_DESCRIPTION = """\
TER (Translation Edit Rate, also called Translation Error Rate) is a metric to quantify the edit operations that a
hypothesis requires to match a reference translation. We use the implementation that is already present in sacrebleu
(https://github.com/mjpost/sacreBLEU#ter), which in turn is inspired by the TERCOM implementation, which can be found
here: https://github.com/jhclark/tercom.
The implementation here is slightly different from sacrebleu in terms of the required input format. The length of
the references and hypotheses lists need to be the same, so you may need to transpose your references compared to
sacrebleu's required input format. See https://github.com/huggingface/datasets/issues/3154#issuecomment-950746534
See the README.md file at https://github.com/mjpost/sacreBLEU#ter for more information.
"""

_KWARGS_DESCRIPTION = """
Produces TER scores alongside the number of edits and reference length.
Args:
    predictions: The system stream (a sequence of segments).
    references: A list of one or more reference streams (each a sequence of segments).
    normalized: Whether to apply basic tokenization to sentences.
    no_punct: Whether to remove punctuations from sentences.
    asian_support: Whether to support Asian character processing.
    case_sensitive: Whether to disable lowercasing.
Returns:
    'score': TER score (num_edits / sum_ref_lengths),
    'num_edits': The cumulative number of edits,
    'ref_length': The cumulative average reference length.
Examples:
    >>> predictions = [["the cat is on the mat", "There is cat playing on the mat"], ["Look! a wonderful day."]]
    >>> references = [
        ["the cat is playing on the mat.", "The cat plays on the mat."], 
        ["Today is a wonderful day", "The weather outside is wonderful."]
    ]
    >>> ter = jury.load_metric("ter")
    >>> results = ter.compute(predictions=predictions, references=references)
    >>> print(results)
    {'ter': {'score': 0.5307692307692308, 'avg_num_edits': 2.75, 'avg_ref_length': 5.75}}
"""


@datasets.utils.file_utils.add_start_docstrings(_DESCRIPTION, _KWARGS_DESCRIPTION)
class TERForLanguageGeneration(MetricForLanguageGeneration):
    def _info(self):
        return datasets.MetricInfo(
            description=_DESCRIPTION,
            citation=_CITATION,
            homepage="http://www.cs.umd.edu/~snover/tercom/",
            inputs_description=_KWARGS_DESCRIPTION,
            features=self._default_features,
            codebase_urls=["https://github.com/mjpost/sacreBLEU#ter"],
            reference_urls=[
                "https://github.com/jhclark/tercom",
            ],
        )

    def _download_and_prepare(self, dl_manager):
        global scb
        global TERScorer

        try:
            import sacrebleu as scb
            from sacrebleu import TER as TERScorer
        except ModuleNotFoundError:
            raise ModuleNotFoundError(requirement_message(path="TER", package_name="sacrebleu"))
        else:
            super(TERForLanguageGeneration, self)._download_and_prepare(dl_manager)

    def _validate_references(self, references: Collator) -> None:
        references_per_prediction = len(references[0])
        if any(len(refs) != references_per_prediction for refs in references):
            raise ValueError("Sacrebleu requires the same number of references for each prediction")

    def _compute_ter_score(
        self, predictions: Sequence[str], references: Sequence[Sequence[str]], sentence_level: bool = False, **kwargs
    ):
        sb_ter = TERScorer(**kwargs)
        if sentence_level:
            output = sb_ter.sentence_score(predictions, references)
        else:
            output = sb_ter.corpus_score(predictions, references)
        return {"score": float(output.score / 100), "num_edits": output.num_edits, "ref_length": output.ref_length}

    def _compute_single_pred_single_ref(
        self,
        predictions: LanguageGenerationInstance,
        references: LanguageGenerationInstance,
        reduce_fn: Callable = None,
        normalized: bool = False,
        no_punct: bool = False,
        asian_support: bool = False,
        case_sensitive: bool = False,
    ):
        return self._compute_ter_score(
            predictions=predictions,
            references=references,
            normalized=normalized,
            no_punct=no_punct,
            asian_support=asian_support,
            case_sensitive=case_sensitive,
        )

    def _compute_single_pred_multi_ref(
        self,
        predictions: LanguageGenerationInstance,
        references: LanguageGenerationInstance,
        reduce_fn: Callable = None,
        normalized: bool = False,
        no_punct: bool = False,
        asian_support: bool = False,
        case_sensitive: bool = False,
    ):
        # SacreBleu inherently supports multiple references.
        return self._compute_ter_score(
            predictions=predictions,
            references=references,
            normalized=normalized,
            no_punct=no_punct,
            asian_support=asian_support,
            case_sensitive=case_sensitive,
        )

    def _compute_multi_pred_multi_ref(
        self,
        predictions: LanguageGenerationInstance,
        references: LanguageGenerationInstance,
        reduce_fn: Callable = None,
        normalized: bool = False,
        no_punct: bool = False,
        asian_support: bool = False,
        case_sensitive: bool = False,
    ):
        scores = []
        avg_num_edits = 0
        avg_ref_length = 0
        for preds, refs in zip(predictions, references):
            pred_scores = []
            num_edits = []
            ref_lengths = []
            for pred in preds:
                score = self._compute_ter_score(
                    predictions=pred,
                    references=refs,
                    sentence_level=True,
                    normalized=normalized,
                    no_punct=no_punct,
                    asian_support=asian_support,
                    case_sensitive=case_sensitive,
                )
                pred_scores.append(score["score"])
                num_edits.append(score["num_edits"])
                ref_lengths.append(score["ref_length"])
            pred_score = reduce_fn(pred_scores).item()
            avg_num_edits += sum(num_edits) / len(num_edits)
            avg_ref_length += sum(ref_lengths) / len(ref_lengths)
            scores.append(pred_score)
        return {
            "score": sum(scores) / len(scores),
            "avg_num_edits": avg_num_edits / len(predictions),
            "avg_ref_length": avg_ref_length / len(predictions),
        }

    def evaluate(
        self, predictions: Collator, references: Collator, reduce_fn: Callable = None, **kwargs
    ) -> Dict[str, float]:
        if predictions.can_collapse() and references.can_collapse():
            predictions = predictions.collapse()
            eval_fn = self._compute_single_pred_single_ref
        elif predictions.can_collapse() and not references.can_collapse():
            predictions = predictions.collapse()
            eval_fn = self._compute_single_pred_multi_ref
        else:
            eval_fn = self._compute_multi_pred_multi_ref
        self._validate_references(references)
        return eval_fn(predictions=predictions, references=references, reduce_fn=reduce_fn, **kwargs)



from jury.metrics._core import MetricAlias
from jury.metrics.bertscore.bertscore_for_language_generation import BertscoreForLanguageGeneration

__main_class__ = "Bertscore"


class Bertscore(MetricAlias):
    _SUBCLASS = BertscoreForLanguageGeneration



import pytest

from jury import Jury
from jury.metrics import AutoMetric
from tests.jury.conftest import get_expected_output
from tests.utils import assert_almost_equal_dict


@pytest.fixture(scope="module")
def jury_chrf():
    metric = AutoMetric.load("chrf")
    return Jury(metrics=metric)


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_basic():
    return output_basic.output


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_multiple_ref():
    return output_multiple_ref.output


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_multiple_pred_multiple_ref():
    return output_multiple_pred_multiple_ref.output


def test_basic(predictions, references, jury_chrf, output_basic):
    scores = jury_chrf(predictions=predictions, references=references)
    assert_almost_equal_dict(actual=scores, desired=output_basic)


def test_multiple_ref(predictions, multiple_references, jury_chrf, output_multiple_ref):
    scores = jury_chrf(predictions=predictions, references=multiple_references)
    assert_almost_equal_dict(actual=scores, desired=output_multiple_ref)


def test_multiple_pred_multiple_ref(
    multiple_predictions, multiple_references, jury_chrf, output_multiple_pred_multiple_ref
):
    scores = jury_chrf(predictions=multiple_predictions, references=multiple_references)
    assert_almost_equal_dict(actual=scores, desired=output_multiple_pred_multiple_ref)



from typing import List, Tuple

from jury.collator import Collator
from jury.utils.nlp import normalize_text


class DefaultTokenizer:
    def tokenize(self, text: str) -> List[str]:
        return normalize_text(text).split()


class TokenizerWrapper:
    """
    Wraps the tokenizer object to adapt tokenize method such that it returns
    a tuple of jury.Collator object instead of list.

    Args:
        tokenizer: Tokenizer object that implements `tokenize` method.
    """

    def __init__(self, tokenizer):
        self.tokenizer = tokenizer

    def tokenize(self, predictions: Collator, references: Collator) -> Tuple[Collator, Collator]:
        _predictions = []
        _references = []
        i = 0
        while i < len(predictions):
            preds = predictions[i]
            refs = references[i]
            _predictions.append([self.tokenizer.tokenize(pred) for pred in preds])
            _references.append([self.tokenizer.tokenize(ref) for ref in refs])
            i += 1
        return Collator(_predictions, keep=True), Collator(_references, keep=True)



import inspect
import json
import os
from typing import Optional

import pytest

from jury import Jury, load_metric
from jury.utils.common import set_env
from tests.jury import EXPECTED_OUTPUTS

# Setting cuda visible devices to None
# assessing CPU only computation for test suite.
set_env("CUDA_VISIBLE_DEVICES", "-1")

_TEST_METRICS = [
    load_metric("accuracy"),
    load_metric("bleu"),
    load_metric("f1"),
    load_metric("meteor"),
    load_metric("precision"),
    load_metric("recall"),
    load_metric("rouge"),
    load_metric("sacrebleu"),
    load_metric("squad"),
]

_STR_TEST_METRIC = "bleu"

_LIST_STR_TEST_METRICS = [
    "accuracy",
    "bleu",
    "f1",
    "meteor",
    "precision",
    "recall",
    "rouge",
    "sacrebleu",
    "squad",
]

_LIST_DICT_TEST_METRICS = [
    {"path": "accuracy"},
    {"path": "bleu", "resulting_name": "bleu-1", "compute_kwargs": {"max_order": 1}},
    {"path": "bleu", "resulting_name": "bleu-2", "compute_kwargs": {"max_order": 2}},
    {"path": "f1", "resulting_name": "F1"},
    {"path": "meteor", "resulting_name": "METEOR"},
    {"path": "precision"},
    {"path": "recall"},
    {"path": "rouge"},
    {"path": "sacrebleu"},
    {"path": "squad"},
]

_LIST_MIXED_TEST_METRICS = [
    "accuracy",
    "bleu",
    {"path": "f1"},
    {"path": "meteor"},
    {"path": "precision"},
    "recall",
    "rouge",
    {"path": "sacrebleu"},
    {"path": "squad"},
]

_DATASETS_METRICS = "competition_math"

_TEST_METRICS_SEQUENCE_CLASSIFICATION = [
    {"path": "accuracy", "task": "sequence-classification"},
    {"path": "f1", "task": "sequence-classification"},
    {"path": "precision", "task": "sequence-classification"},
    {"path": "recall", "task": "sequence-classification"},
]


@pytest.fixture(scope="package")
def predictions():
    return ["There is a cat on the mat.", "Look! a wonderful day."]


@pytest.fixture(scope="package")
def references():
    return ["The cat is playing on the mat.", "Today is a wonderful day"]


@pytest.fixture
def predictions_sequence_classification():
    return [0, 2, 1, 0, 0, 1]


@pytest.fixture
def references_sequence_classification():
    return [0, 1, 2, 0, 1, 2]


@pytest.fixture
def multiple_predictions_sequence_classification():
    return [[0], [1, 2], [0], [1], [0], [1, 2]]


@pytest.fixture
def multiple_references_sequence_classification():
    return [[0, 2], [1, 0], [0, 1], [0], [0], [1, 2]]


@pytest.fixture(scope="function")
def inconsistent_predictions():
    return ["There is a cat on the mat."]


@pytest.fixture(scope="function")
def single_prediction_array():
    return [["the cat is on the mat"], ["Look! a wonderful day."]]


@pytest.fixture(scope="function")
def multiple_predictions_empty():
    return [
        [],
        ["Look! what a wonderful day, today.", "Today is a very wonderful day"],
    ]


@pytest.fixture(scope="function")
def multiple_references_empty():
    return [
        ["the cat is playing on the mat.", "The cat plays on the mat."],
        ["Today is a wonderful day", "The weather outside is wonderful."],
    ]


@pytest.fixture(scope="package")
def multiple_predictions():
    return [
        ["the cat is on the mat", "There is cat playing on mat"],
        ["Look! what a wonderful day, today.", "Today is a very wonderful day"],
    ]


@pytest.fixture(scope="package")
def multiple_references():
    return [
        ["the cat is playing on the mat.", "The cat plays on the mat."],
        ["Today is a wonderful day", "The weather outside is wonderful."],
    ]


@pytest.fixture(scope="module")
def jury_base():
    return Jury(metrics=_TEST_METRICS)


@pytest.fixture(scope="function")
def jury_concurrent():
    return Jury(metrics=_TEST_METRICS, run_concurrent=True)


@pytest.fixture(scope="function")
def jury_str():
    return Jury(metrics=_STR_TEST_METRIC)


@pytest.fixture(scope="function")
def jury_list_str():
    return Jury(metrics=_LIST_STR_TEST_METRICS)


@pytest.fixture(scope="function")
def jury_list_dict():
    return Jury(metrics=_LIST_DICT_TEST_METRICS)


@pytest.fixture(scope="function")
def jury_list_mixed():
    return Jury(metrics=_LIST_MIXED_TEST_METRICS)


@pytest.fixture(scope="function")
def jury_datasets():
    return Jury(metrics=_DATASETS_METRICS)


@pytest.fixture(scope="function")
def jury_sequence_classification():
    return Jury(metrics=_TEST_METRICS_SEQUENCE_CLASSIFICATION)


def get_expected_output(prefix: Optional[str] = None):
    def json_load(path: str):
        with open(path, "r") as jf:
            content = json.load(jf)
        return content

    def wrapper(fn, *args, **kwargs):
        module_name = os.path.basename(inspect.getfile(fn)).replace(".py", "")
        path = os.path.join(EXPECTED_OUTPUTS, prefix, f"{module_name}.json")
        test_name = fn.__name__.replace("output_", "")
        fn.output = json_load(path)[test_name]
        return fn

    if prefix is None:
        prefix = ""
    return wrapper



from jury.metrics._core import MetricAlias
from jury.metrics.bartscore.bartscore_for_language_generation import BartscoreForLanguageGeneration

__main_class__ = "Bartscore"


class Bartscore(MetricAlias):
    _SUBCLASS = BartscoreForLanguageGeneration



from jury.metrics._core import MetricAlias
from jury.metrics.comet.comet_for_language_generation import CometForCrossLingualEvaluation

__main_class__ = "Comet"


class Comet(MetricAlias):
    _SUBCLASS = CometForCrossLingualEvaluation



import re
import string


def normalize_text(text: str, uncased: bool = True) -> str:
    pattern = r"[%s]" % re.escape(string.punctuation)
    text = re.sub(pattern, " ", text)
    normalized_text = " ".join(text.split())
    if uncased:
        normalized_text = normalized_text.lower()
    return normalized_text



from jury.metrics._core import MetricAlias
from jury.metrics.bleurt.bleurt_for_language_generation import BleurtForLanguageGeneration

__main_class__ = "Bleurt"


class Bleurt(MetricAlias):
    _SUBCLASS = BleurtForLanguageGeneration
